@@ -1,5 +1,6 @@
 import numpy as np
-import cupy as cp
+import torch
+import torch.nn.functional as F
 
 def volume_fraction(img, phases={}):
     """
@@ -9,20 +10,20 @@ def volume_fraction(img, phases={}):
     :return: list of volume fractions if no labels, dictionary of label: volume fraction pairs if labelled
     """
 
-    if type(img) is not type(cp.array(1)):
-        img = cp.asarray(img)
+    if type(img) is not type(torch.tensor(1)):
+        img = torch.tensor(img)
 
     if phases=={}:
-        phases = cp.unique(img)
+        phases = torch.unique(img)
         vf_out = []
         for p in phases:
-            vf_out.append((img==p).mean().item())
+            vf_out.append((img==p).to(torch.float).mean().item())
         if len(vf_out)==1:
             vf_out=vf_out[0]
     else:
         vf_out={}
         for p in phases:
-            vf_out[p]=(img==phases[p]).mean().item()
+            vf_out[p]=(img==phases[p]).to(torch.float).mean().item()
 
     return vf_out
 
@@ -31,29 +32,33 @@ def surface_area(img, phases, periodic=False):
     Calculate interfacial surface area between two phases or the total surface area of one phase
     :param img:
     :param phases: list of phases to calculate SA, if lenght 1 calculate total SA, if length 2 calculate inerfacial SA
+    :param periodic: list of bools indicating if the image is periodic in each dimension
     :return: the surface area in faces per unit volume
     """
     shape = img.shape
-    int_not_in_img = np.unique(img).min() -1
+    int_not_in_img = int(np.unique(img).max()+1)
 
     dim = len(shape)
-    img = cp.asarray(img)
+    img = torch.tensor(img)
     # finding an int that is not in the img for padding:
     
-
     if periodic:
-        pad = [(int(not x),int(not x)) for x in periodic]
-        img = cp.pad(img, pad, 'constant', constant_values=int_not_in_img)
+        periodic.reverse()
+        pad = ()
+        for x in periodic:
+            pad += tuple((int(not x),)*2)
+        img = F.pad(img, pad, 'constant', value=int_not_in_img)
+        periodic.reverse()
     else:
-        img = cp.pad(img, 1, 'constant', constant_values=int_not_in_img)
+        img = F.pad(img, (1,)*dim*2, 'constant', value=int_not_in_img)
         periodic=[0]*dim
 
-    SA_map = cp.zeros_like(img)
+    SA_map = torch.zeros_like(img)
     if not isinstance(phases, list):
         phases = [phases]
     for i in range(dim):
         for j in [1, -1]:
-            i_rolled = cp.roll(img, j, i)
+            i_rolled = torch.roll(img, j, i)
             if len(phases)==2:
                 SA_map[(i_rolled == phases[0]) & (img == phases[1])] += 1
             else:
@@ -70,41 +75,41 @@ def surface_area(img, phases, periodic=False):
         z = shape[2]
         if not periodic[2]:
             SA_map = SA_map[:, :, 1:-1]
-        sf = cp.sum(cp.array([x,y,z])[periodic_mask]*cp.roll(cp.array([x,y,z])[periodic_mask],1))
+        sf = torch.sum(torch.tensor([x,y,z])[periodic_mask]*torch.roll(torch.tensor([x,y,z])[periodic_mask],1))
         total_faces = 3*(x*y*z)-sf
     elif dim == 2:
-        sf = cp.sum(cp.array([x,y])[periodic_mask])
+        sf = torch.sum(torch.tensor([x,y])[periodic_mask])
         total_faces = 2*(x+1)*(y+1)-(x+1)-(y+1)-2*sf
     else:
         total_faces=SA_map.size
-    sa = cp.sum(SA_map)/total_faces
+    sa = torch.sum(SA_map)/total_faces
     return sa
 
 def triple_phase_boundary(img):
-    phases = cp.unique(cp.asarray(img))
+    phases = torch.unique(torch.tensor(img))
     if len(phases)!=3:
-        return None
+        raise ValueError('Image must have exactly 3 phases')
     shape = img.shape
     dim = len(shape)
     ph_maps = []
-    img = cp.pad(cp.asarray(img), 1, 'constant', constant_values=-1)
+    img = F.pad(torch.tensor(img), (1,)*dim*2, 'constant', value=-1)
     if dim==2:
         x, y = shape
         total_edges = (x-1)*(y-1)
         for ph in phases:
-            ph_map = cp.zeros_like(img)
-            ph_map_temp = cp.zeros_like(img)
+            ph_map = torch.zeros_like(img)
+            ph_map_temp = torch.zeros_like(img)
             ph_map_temp[img==ph] = 1
             for i in [0, 1]:
                 for j in [0, 1]:
-                    ph_map += cp.roll(cp.roll(ph_map_temp, i, 0), j, 1)
+                    ph_map += torch.roll(torch.roll(ph_map_temp, i, 0), j, 1)
             ph_maps.append(ph_map)
-        tpb_map = cp.ones_like(img)
+        tpb_map = torch.ones_like(img)
         for ph_map in ph_maps:
             tpb_map *= ph_map
         tpb_map[tpb_map>1] = 1
         tpb_map = tpb_map[1:-1, 1:-1]
-        tpb = np.sum(tpb_map)
+        tpb = torch.sum(tpb_map)
     else:
         tpb = 0
         x, y, z = shape
@@ -113,20 +118,20 @@ def triple_phase_boundary(img):
         for d in range(dim):
             ph_maps = []
             for ph in phases:
-                ph_map = cp.zeros_like(img)
-                ph_map_temp = cp.zeros_like(img)
+                ph_map = torch.zeros_like(img)
+                ph_map_temp = torch.zeros_like(img)
                 ph_map_temp[img==ph] = 1
                 for i in [0, 1]:
                     for j in [0, 1]:
                         d1 =( d + 1) % 3
                         d2 = (d + 2) % 3
-                        ph_map += cp.roll(cp.roll(ph_map_temp, i, d1), j, d2)
+                        ph_map += torch.roll(torch.roll(ph_map_temp, i, d1), j, d2)
                 ph_maps.append(ph_map)
-            tpb_map = cp.ones_like(img)
+            tpb_map = torch.ones_like(img)
             for ph_map in ph_maps:
                 tpb_map *= ph_map
             tpb_map[tpb_map>1] = 1
             tpb_map = tpb_map[1:-1, 1:-1, 1:-1]
-            tpb += np.sum(tpb_map)
+            tpb += torch.sum(tpb_map)
 
     return tpb/total_edges
